/******************************************************************************************

CreateServiceRequest.java

The purpose of this servlet is to create service requests, regardless of which jsp the
service request originated.

   + This servlet is invoked by CreateServiceRequest.jsp or CreateServiceRequest_Technician.jsp
   + This servlet dispatches to MainPage_Client.jsp or MainPage.jsp
   
******************************************************************************************/

package CTTS;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import CTTS.DBI;
import CTTS.Shared;

public class CreateServiceRequest extends CTTS.Control
{
   protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      HttpSession session = request.getSession(); // returns the current session associated with this request, or if the request does not have a session, creates one.
      CTTS.Shared bean = (CTTS.Shared)session.getAttribute("shared"); // get bean from the session
       
		/* gets user inputs POSTed from the HttpServletRequest, request */
		String personReported = request.getParameter("requestor"); 
      String clientId = request.getParameter("clientId"); 
		String equipId = request.getParameter("equipId"); 
		String problemDescription = request.getParameter("problemDescription"); 
      
      if (validateUserInput(bean, clientId, equipId)) // call validateUserInput method and pass the bean client id and equipment id to ensure input is valid
      {
         submitServiceRequest(bean, personReported, clientId, equipId, problemDescription);
         
         if (bean.getUserIsEmployee())
            this.gotoPage("/MainPage.jsp", request,response); // dispatcher to MainPage.jsp
         else
            this.gotoPage("/MainPage_Client.jsp", request,response); // dispatcher to MainPage_Client.jsp
      }
      else // selected equipment or client ID does not belong to user or does not exist.
      {
         if (bean.getUserIsEmployee())
            this.gotoPage("/CreateServiceRequest_Technician.jsp", request,response); // dispatcher to CreateServiceRequest_Technician.jsp
         else
            this.gotoPage("/CreateServiceRequest.jsp", request,response); // dispatcher to CreateServiceRequest.jsp
      }
   }
   
   private boolean validateUserInput(CTTS.Shared bean, String clientId, String equipId)
   {  
      CTTS.DBI dbi = new CTTS.DBI(); // new DBI object to access the database. DBI (Data Base Interface)
      
      try
      {
         if(dbi.connect())
         {
            if (dbi.clientExists(clientId))
            {
               if (dbi.equipIdBelongsToRequestorCompany(clientId, equipId))
               {
                  return true;
               }
               else
               {
                  bean.setError("The provided equipment ID is not recognized; it may not exist or does not belong to the provided client ID. Please enter a valid equipment ID.");
                  return false;
               }
            }
            else
            {
               bean.setError("The provided client ID is not recognized. Please enter a valid client ID.");
               return false;
            }
         }
         else
         {
            return false;
         }
      }
      catch(Exception e)
      {
         e.printStackTrace();
         bean.setMessage("Exception caught : " + e);
         bean.setError("The database could not be connected");
         return false;
      }
      finally
      {
         dbi.close();  //return connection to Tomcat
      }
   }
   
   private void submitServiceRequest(CTTS.Shared bean, String requestor, String clientId, String equipId, String problemDescription) throws ServletException, IOException
   {
      CTTS.DBI dbi = new CTTS.DBI();  // new DBI object to access the database. DBI (Data Base Interface)
      
      try
      {
         dbi = new CTTS.DBI();   //creates a new instance of dbi
         if(dbi.connect())
         {
            dbi.createServiceRequest(requestor, clientId, equipId, problemDescription);
         }
      }
      catch(Exception e)
      {
         e.printStackTrace();
         bean.setMessage("Exception caught : " + e);
         bean.setError("The database could not be connected");
      }
      finally
      {
         dbi.close();  //return connection to Tomcat
      }
   }
}
