/******************************************************************************************

UpdateServiceRequest.java

The purpose of this servlet is to 

   + This servlet is invoked by ViewServiceRequest.jsp
   + This servlet dispatches to ViewServiceRequest.java or ViewMyServiceRequests.java
   
******************************************************************************************/


package CTTS;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import CTTS.DBI;
import CTTS.Shared;

public class UpdateServiceRequest extends CTTS.Control
{
   boolean ticketEscalatedToAnyone = false; // represents whether or not the ticket it escalated to the entire team (true) vs another team member else specifically (false).

   protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      HttpSession session = request.getSession(); // returns the current session associated with this request, or if the request does not have a session, creates one.
      CTTS.Shared bean = (CTTS.Shared)session.getAttribute("shared"); // get bean from the session
		
      /* gets only the needed user inputs POSTed from the HttpServletRequest, request */
		String originalStatus = request.getParameter("originalStatus");
      String status = request.getParameter("status");
      String originalTechnician = request.getParameter("originalTechnician"); 
      String assignedTechnician = request.getParameter("assignedTechnician"); 
      String startTime = request.getParameter("startTime");
      String finishTime = request.getParameter("finishTime");
      String description = request.getParameter("description"); 
      String srId = request.getParameter("srId");
      
      /* create some variables to track changes within the service request */
      boolean submitServiceWork = false;
      boolean changeAssignedTechnician = false;
      boolean changeStatus = false;
      boolean ticketEscalatedToAnyone = false;
      
      // Did the technician try to add service work?
      if ((!startTime.isEmpty() && startTime!=null) && (!finishTime.isEmpty() && finishTime!=null) && (!description.isEmpty() && description!=null)) // if all service work was entered...      
      {
         submitServiceWork = true;
      }
      else if ((startTime.isEmpty() || startTime==null) && (finishTime.isEmpty() || finishTime==null) && (description.isEmpty() || description==null)) // if user did not try to add service work...
      {
         // do nothing
      }
      else // user entered some, but not all required info for service work
      {
         bean.setError("You did not enter all of the needed information in order to submit service work. Please provide a start time, finish time, and description."); // notify user they need to enter needed info
      }
      
      if (!assignedTechnician.equals(originalTechnician)) // Was the assigned technician changed?
         changeAssignedTechnician = true;
      
      if (!status.equals(originalStatus)) // Has the status changed?
         changeStatus = true;
      
      if (submitServiceWork || changeAssignedTechnician || changeStatus) // something has changed, update the service request
         updateServiceRequest(bean, srId, status, assignedTechnician, startTime, finishTime, description, submitServiceWork, changeAssignedTechnician, changeStatus); // call updateServiceRequest function and pass all inputs.
      
      /* Dispatches to original "Control" servlet */
      //bean.setMessage("status: "+status+", assignedTechnician: "+assignedTechnician+", startTime: "+startTime+", finishTime: "+finishTime+", description: "+description); // debug message
      if (ticketEscalatedToAnyone == true)
         this.gotoPage("/ViewServiceRequest", request,response); // dispatcher to ViewServiceRequest.java
      else
         this.gotoPage("/ViewMyServiceRequests", request,response); // dispatcher to ViewMyServiceRequests.java
   }
   
   private void updateServiceRequest(CTTS.Shared bean, String srId, String status, String assignedTechnician, String startTime, String finishTime, String description, boolean submitServiceWork, boolean changeAssignedTechnician, boolean changeStatus) throws ServletException, IOException
   {
      CTTS.DBI dbi = new CTTS.DBI();  // create new DBI object to access the database. DBI (Data Base Interface)
      
      try
      {
         if(dbi.connect())
         {
            int empId = bean.getEmpId(); // get current employee ID
            if (submitServiceWork) // if user attempted to enter service work with all requirements
               dbi.addServiceWork(srId, empId, startTime, finishTime, description);
            if (changeAssignedTechnician) // if user changed the assigned technician
               dbi.escalateServiceRequest(srId, assignedTechnician);
            if (changeStatus) // if user tried to change the status of the service request
               dbi.updateServiceRequestStatus(srId, status);
            if (changeStatus==true && changeAssignedTechnician==false && status.equals("Escalated")) // if user tried to escalate the ticket, but did not specify who it is escalated to
            {
               dbi.unassignTechnician(srId); // remove the assigned technciian from the service request
               ticketEscalatedToAnyone = true; // set ticket escalated to anyone as true so the employee will be directed to MyServiceRequests
            }
         }
      }
      catch(Exception e)
      {
         e.printStackTrace();
         bean.setMessage("Exception caught : " + e);
         bean.setError("The database could not be connected");
      }
      finally
      {
         dbi.close();  //return connection to Tomcat
      }
   }
}
